@@ -6,6 +6,8 @@ import logging
 from datetime import datetime, timedelta
 from typing import Dict, List, Optional, Tuple
 from . import typings
+from .software_version import SoftwareVersion
+import traceback
 
 import asyncio
 import aiohttp
@@ -22,6 +24,9 @@ DEFAULT_RTMP_AUTH_METHOD = 'PASSWORD'
 _LOGGER = logging.getLogger(__name__)
 
 
+ref_sw_version_3_0_0_0_0 = SoftwareVersion("v3.0.0.0_0")
+
+
 class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-public-methods
     """Reolink API class."""
 
@@ -58,6 +63,7 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
         self._device_info = None
         self._hdd_info = None
         self._ftp_state = None
+        self._push_state = None
         self._email_state = None
         self._ir_state = None
         self._daynight_state = None
@@ -71,6 +77,7 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
         self._serial = None
         self._name = None
         self._sw_version = None
+        self._sw_version_object: Optional[SoftwareVersion] = None
         self._model = None
         self._channels = None
         self._ptz_presets = dict()
@@ -79,6 +86,7 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
 
         self._isp_settings = None
         self._ftp_settings = None
+        self._push_settings = None
         self._enc_settings = None
         self._ptz_presets_settings = None
         self._ability_settings = None
@@ -157,6 +165,11 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
         """Return the FTP state."""
         return self._ftp_state
 
+    @property
+    def push_state(self):
+        """Return the PUSH (notifications) state."""
+        return self._push_state
+
     @property
     def email_state(self):
         """Return the email state."""
@@ -269,6 +282,9 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
         if self._ftp_state is not None:
             capabilities.append("ftp")
 
+        if self._push_state is not None:
+            capabilities.append("push")
+
         if self._ir_state is not None:
             capabilities.append("irLights")
 
@@ -319,6 +335,11 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
             },
         ]
 
+        if self._sw_version_object is not None and self._sw_version_object > ref_sw_version_3_0_0_0_0:
+            body.append({"cmd": "GetPushV20", "action": 1, "param": {"channel": self._channel}})
+        else:
+            body.append({"cmd": "GetPush", "action": 1, "param": {"channel": self._channel}})
+
         if cmd_list is not None:
             for x, line in enumerate(body):
                 if line["cmd"] not in cmd_list:
@@ -478,10 +499,11 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
                 elif data["cmd"] == "GetDevInfo":
                     self._device_info = data
                     self._serial = data["value"]["DevInfo"]["serial"]
-                    self._device_name = data["value"]["DevInfo"]["name"]
+                    self._name = data["value"]["DevInfo"]["name"]
                     self._sw_version = data["value"]["DevInfo"]["firmVer"]
                     self._model = data["value"]["DevInfo"]["model"]
                     self._channels = data["value"]["DevInfo"]["channelNum"]
+                    self._sw_version_object = SoftwareVersion(self._sw_version)
 
                 elif data["cmd"] == "GetHddInfo":
                     self._hdd_info = data["value"]["HddInfo"]
@@ -507,6 +529,14 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
                     self._ftp_settings = data
                     self._ftp_state = data["value"]["Ftp"]["schedule"]["enable"] == 1
 
+                elif data["cmd"] == "GetPush":
+                    self._push_settings = data
+                    self._push_state = data["value"]["Push"]["schedule"]["enable"] == 1
+
+                elif data["cmd"] == "GetPushV20":
+                    self._push_settings = data
+                    self._push_state = data["value"]["Push"]["enable"] == 1
+
                 elif data["cmd"] == "GetEnc":
                     self._enc_settings = data
                     self._audio_state = data["value"]["Enc"]["audio"] == 1
@@ -548,7 +578,9 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
                 elif data["cmd"] == "GetAbility":
                     for ability in data["value"]["Ability"]["abilityChn"]:
                         self._ptz_support = ability["ptzCtrl"]["permit"] != 0
+
             except:  # pylint: disable=bare-except
+                _LOGGER.error(traceback.format_exc())
                 continue
 
     async def login(self):
@@ -669,6 +701,26 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
 
         return await self.send_setting(body)
 
+    async def set_push(self, enable: bool):
+        """Set the PUSH (notifications) parameter."""
+        if not self._push_settings:
+            _LOGGER.error("Actual PUSH settings not available")
+            return False
+
+        if enable:
+            new_value = 1
+        else:
+            new_value = 0
+
+        if self._sw_version_object is not None and self._sw_version_object > ref_sw_version_3_0_0_0_0:
+            body = [{"cmd": "SetPushV20", "action": 0, "param": self._push_settings["value"]}]
+            body[0]["param"]["Push"]["enable"] = new_value
+        else:
+            body = [{"cmd": "SetPush", "action": 0, "param": self._push_settings["value"]}]
+            body[0]["param"]["Push"]["schedule"]["enable"] = new_value
+
+        return await self.send_setting(body)
+
     async def set_audio(self, enable):
         """Set the audio parameter."""
         if not self._enc_settings:
